//-----------------------------------------------------------------------------
// File: SkinnedMesh-Mesh.h
//
// Desc: CD3DSkinMesh definition
//
// Copyright (c) Microsoft Corporation. All rights reserved.
//-----------------------------------------------------------------------------
 
// enum for various skinning modes possible
enum METHOD 
{
    D3DNONINDEXED,
    D3DINDEXED,
    SOFTWARE,
    D3DINDEXEDVS,
    D3DINDEXEDHLSLVS,
    NONE
};




//-----------------------------------------------------------------------------
// Name: struct D3DXFRAME_DERIVED
// Desc: Structure derived from D3DXFRAME so we can add some app-specific
//       info that will be stored with each frame
//-----------------------------------------------------------------------------
struct D3DXFRAME_DERIVED: public D3DXFRAME
{
    D3DXMATRIXA16        CombinedTransformationMatrix;
};




//-----------------------------------------------------------------------------
// Name: struct D3DXMESHCONTAINER_DERIVED
// Desc: Structure derived from D3DXMESHCONTAINER so we can add some app-specific
//       info that will be stored with each mesh
//-----------------------------------------------------------------------------
struct D3DXMESHCONTAINER_DERIVED: public D3DXMESHCONTAINER
{
    LPDIRECT3DTEXTURE9*  ppTextures;       // array of textures, entries are NULL if no texture specified    
                                
    // SkinMesh info             
    LPD3DXMESH           pOrigMesh;
    LPD3DXATTRIBUTERANGE pAttributeTable;
    DWORD                NumAttributeGroups; 
    DWORD                NumInfl;
    LPD3DXBUFFER         pBoneCombinationBuf;
    D3DXMATRIXA16**         ppBoneMatrixPtrs;
    D3DXMATRIXA16*          pBoneOffsetMatrices;
    DWORD                NumPaletteEntries;
    bool                 UseSoftwareVP;
    DWORD                iAttributeSW;     // used to denote the split between SW and HW if necessary for non-indexed skinning
};




//-----------------------------------------------------------------------------
// Name: class CD3DSkinMesh
// Desc: skinned mesh class. 
//-----------------------------------------------------------------------------
class CD3DSkinMesh
{    
public:
    CD3DSkinMesh();

	//CD3DApp mini-API
    HRESULT OneTimeSceneInit();
    HRESULT InitDeviceObjects(TCHAR*            strMeshPath,
							  TCHAR*            strSkinnedMeshPath,
							  D3DCAPS9          d3dCaps,
							  LPDIRECT3DDEVICE9 pd3dDevice,
							  D3DXVECTOR3*      vObjectCenter,
							  FLOAT*            fObjectRadius);
    HRESULT RestoreDeviceObjects(LPDIRECT3DDEVICE9 pd3dDevice);
    HRESULT FrameMove(const LPD3DXMATRIX pmatWorld, float fElapsedTime);
    HRESULT Render(D3DCAPS9 d3dCaps,LPDIRECT3DDEVICE9 pd3dDevice,
		           const LPD3DXMATRIX pmatView, const LPD3DXMATRIX pmatProj,
				   D3DXVECTOR4 vLightDir);
    HRESULT InvalidateDeviceObjects();
    HRESULT DeleteDeviceObjects(D3DCAPS9 d3dCaps);
    HRESULT FinalCleanup();

	//public helpers
	METHOD       getSkinningMethod() { return m_SkinningMethod; };
	LPD3DXFRAME  getFrameRoot() { return m_pFrameRoot; };

    HRESULT GenerateSkinnedMesh( D3DXMESHCONTAINER_DERIVED *pMeshContainer,
								 D3DCAPS9                   d3dCaps,           
                                 LPDIRECT3DDEVICE9          pd3dDevice,   
	                             METHOD                     SkinningMethod );
	
    void UpdateSkinningMethod( LPD3DXFRAME       pFrameBase,
							   D3DCAPS9          d3dCaps,   
							   LPDIRECT3DDEVICE9 pd3dDevice, 
							   METHOD            SkinningMethod );

protected:
	LPD3DXFRAME                 m_pFrameRoot;
    LPD3DXANIMATIONCONTROLLER   m_pAnimController;    

	METHOD                      m_SkinningMethod;   // Current skinning method
    D3DXMATRIXA16*              m_pBoneMatrices;
    UINT                        m_NumBoneMatricesMax;

    LPDIRECT3DVERTEXSHADER9     m_pIndexedVertexShader[4];
    LPD3DXEFFECT                m_pEffect;
	
    void DrawMeshContainer( LPD3DXMESHCONTAINER pMeshContainerBase, LPD3DXFRAME pFrameBase,
		                    D3DCAPS9 d3dCaps,LPDIRECT3DDEVICE9 pd3dDevice, const LPD3DXMATRIX pmatView );
    void DrawFrame( LPD3DXFRAME       pFrame,
		            D3DCAPS9          d3dCaps,
					LPDIRECT3DDEVICE9 pd3dDevice,
					const LPD3DXMATRIX        pmatView );

    HRESULT SetupBoneMatrixPointersOnMesh( LPD3DXMESHCONTAINER pMeshContainer );
    HRESULT SetupBoneMatrixPointers( LPD3DXFRAME pFrame );
    void UpdateFrameMatrices( LPD3DXFRAME pFrameBase, const LPD3DXMATRIX pParentMatrix );
};
